import os
import octofuss
import octofuss.samba as osmb
from urllib.parse import quote_plus, unquote_plus

class Samba(octofuss.Tree):
    keyword_doc = {
        "path": "path to be shared",
        "readonly": "if true, this share will not allow writing",
        "guestok": "if true, this share can be used by anyone",
        "readlist": "groups that are allowed to read from this share",
        "writelist": "groups that are allowed to write to this share",
    }

    def __init__(self, name, filename=None):
        """
        name: tree name
        filename: the configuration file name. If None, a mock, in memory
                  configuration is used
        """
        super(Samba, self).__init__(name, doc="Configuration of the Samba server")
        self.filename = filename
        self.conf = None
        self.conf_ts = 0

    def _checkConfig(self):
        if self.filename != None:
            mtime = os.stat(self.filename).st_mtime
            if self.conf_ts == 0 or mtime > self.conf_ts:
                self.conf = osmb.SambaConf()
                self.conf.ReadConfig(self.filename)
                self.conf_ts = mtime
        elif self.conf_ts == 0:
            from io import StringIO
            self.conf = osmb.SambaConf()
            self.conf.ReadConfig("mock.conf", dummyfd=StringIO("""
            [global]
            workgroup=test

            [cdrom]
            path=/cdrom
            read only=yes
            guest ok=yes
            write list=@wizards

            [public]
            path=/public
            read only=no
            guest ok=no

            [news area]
            path=/news
            read only=yes
            write list=@editors

            [cabal]
            path=/cabal
            read only=yes
            read list=@cabal
            write list=@cabal

            [PRINT$]
            comment = Printer Drivers
            path = /var/lib/samba/printers
            browseable = yes
            read only = yes
            guest ok = no
            """))
            self.conf_ts = 1

    def _commit(self):
        if self.filename != None:
            self.conf.Write(self.filename)

    def _service(self, name):
        """
        Return a dict of key=value for a smb.conf section, with
        lowercased section name and keys
        """
        res = self.conf.getService(name)
        return dict([(x.lower(), y.getValue()) for x, y in list(res.items())])

    def lhas(self, path):
        self._checkConfig()
        if len(path) == 0: return True
        name = unquote_plus(path[0])
        if not self.conf.isService(name): return False
        if len(path) == 1: return True
        if len(path) > 2: return False
        return path[1].upper() in self.conf.getService(name)

    def lget(self, path):
        self._checkConfig()
        if len(path) == 0:
            # Convert service names and keys to lower case
            res = dict()
            for sname in self.conf.Services():
                res[sname.lower()] = self._service(sname)
            return res
        name = unquote_plus(path[0])
        if not self.conf.isService(name): return False
        if len(path) > 2: return None
        sdict = self._service(name)
        if len(path) == 1: return sdict
        return sdict.get(path[1], None)

    def llist(self, path):
        self._checkConfig()
        if len(path) == 0: return sorted([quote_plus(x.lower()) for x in self.conf.Services()])
        name = unquote_plus(path[0])
        if not self.conf.isService(name): return []
        if len(path) > 1: return []
        return sorted([quote_plus(x.lower()) for x in self.conf.getService(name)])

    def lset(self, path, value):
        self._checkConfig()
        if len(path) == 0 or len(path) > 2:
            return super(Samba, self).lset(path, value)
        name = unquote_plus(path[0])
        if not self.conf.isService(unquote_plus(name)):
            return super(Samba, self).lset(path, value)
        if len(path) == 1:
            for k, v in list(value.items()):
                self.conf.SetServiceOption(name, k, v)
            self._commit()
        else:
            self.conf.SetServiceOption(name, path[1], value)
            self._commit()

    def lcreate(self, path, value):
        self._checkConfig()
        if len(path) == 0 or len(path) > 2:
            return super(Samba, self).lcreate(path, value)
        name = unquote_plus(path[0])
        if not self.conf.isService(name):
            self.conf.AddService(name)
        if len(path) == 1:
            if value == None or value == "": value = {}
            for k, v in list(value.items()):
                self.conf.SetServiceOption(name, k, v)
            self._commit()
        else:
            self.conf.SetServiceOption(name, path[1], value)
            self._commit()

    def ldelete(self, path):
        self._checkConfig()
        if len(path) == 0 or len(path) > 2:
            return super(Samba, self).ldelete(path)
        name = unquote_plus(path[0])
        if not self.conf.isService(name):
            return super(Samba, self).ldelete(path)
        if len(path) == 1:
            self.conf.DelService(name)
            self._commit()
        else:
            # TODO: how do we delete a key from a service?
            pass

    def ldoc(self, path):
        self._checkConfig()
        if len(path) == 0: return super(Samba, self).ldoc(path)
        name = unquote_plus(path[0])
        if not self.conf.isService(name): return None
        if len(path) > 2: return None
        if len(path) == 1:
            if name == "global":
                return "Global samba configuration"
            else:
                return "Configuration for share %s" % unquote_plus(name)
        return self.keyword_doc.get(path[1], None)


def init(**kw):
    conf = kw.get("conf", None)
    canMock = conf and conf.getboolean("General", "allowMockup")
    forceMock = conf and conf.getboolean("General", "forceMockup")
    fname = None
    if not forceMock and os.path.exists("/etc/samba/smb.conf"):
        fname = "/etc/samba/smb.conf"
    elif forceMock or canMock:
        fname = None
    yield dict(tree = Samba("samba", fname), root="/")
